package dmc;

/*
 * TabbedChat.java
 * Matthew Wronka <wronkm@rpi.edu>
 *
 * Tabbed dialog for all active chats on a connection.
 * Also includes callbacks which implement RMsgHandler for use
 * with conversations on this connection.
 *
 * Copyright (C) 2002 Matthew Wronka <wronkm@rpi.edu>
 * This file is released under the GNU General Public License version 2,
 * a version of which should have been provided along with this file.
 *
 * You might also find a copy at:
 * http://dmc.sf.net/files/gpl.txt
 * http://www.fsf.org/copyleft/gpl.html
 */


import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.net.*;
import java.util.LinkedList;
import java.util.Date;

import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import wtalk.WTalk;
import wtalk.RMsgHandler;




/**
 * Handles all updating when a message is written.
 * 	Makes things happy for swing.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
class ChatUpdater implements Runnable {
	protected JTextPane tp;
	protected String str;
	protected Style style;

	public ChatUpdater(JTextPane tp_, String str_, Style style_) {
		tp=tp_; str=str_; style=style_;
	}

	public void run() {
		Document doc = tp.getDocument();

		try { 
			doc.insertString(doc.getLength(),str, style);
			tp.scrollRectToVisible(new Rectangle(0, tp.getHeight(), 1, 1));
		} catch (BadLocationException ble) {}
	}

}

/**
 * Maintains logging functionality.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
class ChatLogger implements Runnable {
	TabbedChat tc;
	String sFrom;
	String sChat;
	String sText;

	public ChatLogger(TabbedChat tc_, String from_, String chat_, String text_) {
		tc = tc_; sFrom = from_; sChat = chat_; sText = text_;
	}

	private String getLogFile() {
		// should be of form $HOME/.dmc/$CONNECTION/$CHAT
		String rval="";
		String sFSep =  System.getProperty("file.separator");

		rval += System.getProperty("user.home");
		rval += sFSep;
		rval += ".dmc";
		rval += sFSep;
		rval += tc.getConnectionName();

		(new java.io.File(rval)).mkdirs();

		rval += sFSep;
		rval += sChat;

		return rval;
	}

	private String getOutput() {
		String rval;
		rval =  new Date() + "\t" + sFrom + "\t" + sText + 
				System.getProperty("line.separator");
		return rval;
	}

	public void run() {

		try {
			FileOutputStream fos = new FileOutputStream
					(getLogFile(), true);

			fos.write(getOutput().getBytes());

			fos.close();	
		} catch (FileNotFoundException eFNF) { 
		} catch (IOException eIO) { 
		}
	}

}



/**
 * Handles highlighting of tabs.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
class TabHighlighter implements Runnable {
	protected String tab;
	protected TabbedChat tc;

	public TabHighlighter(TabbedChat tc_, String tab_) {
		tc = tc_;	tab = tab_;
	}

	public void run() {
		// Highlight
		int i=tc.indexOfTab(tab);
		if(i!=tc.getUI().getSelectedIndex()){
			tc.getUI().setForegroundAt(i, DmcColors.TAB_HIGHLIGHT);
		}

		// Highlight the connection tab
		if(tc.getTabbedConnection() != null) {
			i=tc.getTabbedConnection().indexOfTab
					(tc.getConnectionName());
			if(i!=tc.getTabbedConnection().getUI
					().getSelectedIndex()){
				tc.getTabbedConnection().getUI
						().setForegroundAt
						(i, DmcColors.TAB_HIGHLIGHT);
			}
		}

	}

}


/**
 * Handles all (group) chat messages.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
class ChatReader implements RMsgHandler {
	protected TabbedChat tc;

	/**
	 * Default ctor.
	 *
	 */
	public ChatReader(){}

        /**
         * Sets the parent TabbedChat object.   
         *
         * @param t Parent TabbedChat object.
         */
	public void setTabs(TabbedChat t){
		tc = t;
	}

	/**
	 * Implements handle to handle received messages.
	 *
	 * @param to The message target.
	 * @param from The message sender.
	 * @param data The message contents.
	 */
	public void handle(String to, String from, String data) {
		JTextPane tp;
		TabHighlighter th;
		String str;
		ChatUpdater cu;
		ChatLogger cl;
		boolean isReceiver = (from != tc.getUserName()); 	// Receiver or Sender?

		// If the sender is being ignored for CHAT, leave now!
		if(tc.isIgnored(from, "CHAT")) return;

		int i = tc.indexOfTab(to);	// Why this twice?
		tp=tc.openBuddy(to);

		th = new TabHighlighter(tc, to);
		SwingUtilities.invokeLater(th);

		if(isReceiver) {
			Style myStyle = tp.getStyle("usay");
			StyleConstants.setForeground( myStyle, 
				DmcColors.colorNick(from) );

			str = "<" + from + "> ";
			cu = new ChatUpdater(tp, str, myStyle);
			SwingUtilities.invokeLater(cu);
		} else {
			str = "<" + from + "> ";
			cu = new ChatUpdater(tp, str, 
					tp.getStyle("isay"));
			SwingUtilities.invokeLater(cu);
		}

		cu = new ChatUpdater(tp, data + "\n", 
				tp.getStyle("chat"));

		cl = new ChatLogger(tc, from, to, data);
		SwingUtilities.invokeLater(cu);
		SwingUtilities.invokeLater(cl);
 
		System.out.println("[" +to+ "] <" + from + "> " +data);


	}
}

/**
 * Handles all brodcase (wall) messages.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
class WallReader implements RMsgHandler {
	private TabbedChat tc;
	private JTextPane tp;

        /**
         * Default ctor.
         *
         */
	public WallReader(){}

	/**
	 * Overloaded ctor to set JTextPane to use.
	 *
	 * @param t JTextPane to use when displaying walls.
	 */
	public WallReader(JTextPane t){
		setPane(t);
	}

        /**
         * Sets the parent TabbedChat object.   
         *
         * @param t Parent TabbedChat object.
         */
	public void setPane(JTextPane t){
		tp = t; 
	}

        /**
         * Sets the parent TabbedChat object.
         *
         * @param t Parent TabbedChat object.
         */
        public void setTabs(TabbedChat t){
                tc = t;
        }

        /**
         * Implements handle to handle received messages.
         *
         * @param to The message target. This may be null.
         * @param from The message sender.
         * @param data The message contents.
         */
	public void handle(String to, String from, String data) {
		ChatUpdater cu;
		ChatLogger cl;


		// If someone set a TabbedChat, and we're ignoring WALLs from
		// 	the sender, leave now!
		if(tc!=null)
			if(tc.isIgnored(from, "WALL")) return;


		cu = new ChatUpdater(tp, 
				from + ": " + data + "\n", tp.getStyle("wall"));
		cl = new ChatLogger(tc, from, "SYSTEM", data);
		SwingUtilities.invokeLater(cu);
		SwingUtilities.invokeLater(cl);
	}
}

/**
 * Handles all system messages.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
class SystemReader implements RMsgHandler {
	protected TabbedChat tc;
	private JTextPane tp;

        /**
         * Default ctor.
         *
         */
	public SystemReader(){}

        /**
         * Overloaded ctor to set TabbedChat and JTextPane to use.
         *
         * @param t1 Parent TabbedChat object.
         * @param t2 JTextPane to use when displaying walls.
         */
	public SystemReader(TabbedChat t1, JTextPane t2){
		tc = t1;
		setPane(t2);
	}

        /**
         * Sets the parent TabbedChat object.   
         *
         * @param t Parent TabbedChat object.
         */
	public void setTabs(TabbedChat t){
		tc = t;
	}

	/**
	 * Sets the default JTextPane.
	 *
	 * @param t Default JTextPane for system messages.
	 */
	public void setPane(JTextPane t){
		tp = t; 
	}

        /**
         * Implements handle to handle received messages.
         *
         * @param to The message target. This may be null.
         * @param from The message sender. This may be null.
         * @param data The message contents.
         */
	public void handle(String to, String from, String data) {
		JTextPane targetTp;
		TabHighlighter th;
		ChatUpdater cu;
		ChatLogger cl;
		boolean useSystem = (to==null || to.equals(""));
		int i;


		// If we are ignoring System messages from the sender, leave!
		if(tc.isIgnored(from, "SYSTEM")) return;


		if(!useSystem) {
			i = tc.indexOfTab(to);
			if( i < 0) useSystem= true;
		}

		if(!useSystem) {
			targetTp = tc.openBuddy(to);

			th = new TabHighlighter(tc, to);
			SwingUtilities.invokeLater(th);

		} else {
			targetTp = tp;
		}

		cu = new ChatUpdater(targetTp, "-- " + data + "\n", 
				targetTp.getStyle("server"));
		cl = new ChatLogger(tc, from, useSystem?"SYSTEM":to, data);
		SwingUtilities.invokeLater(cu);
		SwingUtilities.invokeLater(cl);
	}
}

/**
 * Handles all private (person to person)  messages.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
class MsgReader implements RMsgHandler {
	protected TabbedChat tc;

        /**
         * Default ctor.
         *
         */
	public MsgReader(){}

        /**
         * Sets the parent TabbedChat object.   
         *
         * @param t Parent TabbedChat object.
         */
	public void setTabs(TabbedChat t){ 
		tc = t;
	}

        /**
         * Implements handle to handle received messages.
	 * This class also supports client->client encryption of message data.
         *
         * @param to The message target.
         * @param from The message sender.  This may be null when this class
	 *	is being used to echo outgoig messages, such as in the case
	 *	where the user does not receive messages sent.
         * @param data The message contents.
         */
	public void handle(String to, String from, String data){
		JTextPane tp;
		TabHighlighter th;
		ChatUpdater cu;
		ChatLogger cl;
		String ciphertext=data;
		String keytype = null;
		String cipher=null;
		java.util.StringTokenizer st = new java.util.StringTokenizer(data);
		boolean isReceiver = (from != null);	//(from != tc.getWTalkProtocol().getName()); 	// Receiver or Sender?
		String s = isReceiver? from : to;


		// If we're ignoring MSGs from this person, leave!
		if(tc.isIgnored(from, "MSG")) return;






		int i = tc.indexOfTab(s);

		if(i>=0)
			tp = ((JChatPanel)(tc.getUI().getComponentAt(i))).getJTextPane();
		else
			tp = tc.addTab(s, JChatPanel.IM_TAB).getJTextPane();

		i = tc.indexOfTab(s);
		ChatInputHandler ci = ((JChatPanel)(tc.getUI().getComponentAt(i))).getChatInputHandler();


		// They want us to send our public key
		if(data.charAt(0)==1){
			//System.out.println("1...");
			ci.sendPublicKey();
			return;
		}


		if(data.charAt(0)==2){
			//System.out.println("2...");
			// Did we get a key?
			if(st.hasMoreTokens() ) keytype = st.nextToken();
			//if( keytype.equals("dmc-key")) {
				if(isReceiver && st.hasMoreTokens()){
					ci.setPublicKey(data.substring(keytype.length()+1));
					ci.negotiateSecure();
				}
				//return;
			//}
			return;
		}


		if(data.charAt(0)==3) {
			//System.out.println("3...");
			ciphertext =ci.decryptRSA(data.substring(1));
			//st = new java.util.StringTokenizer(ciphertext);
			//if(st.hasMoreTokens() ) cipher= st.nextToken();
			// changing encryption protocols?
			if( isReceiver ){
				ci.manageSecure("blowfish");
				ci.setSharedKey(ciphertext.getBytes());
				//ci.manageSecure(cipher);
				//System.out.println("New cipher: " + cipher);
				//if(ci.manageSecure("blowfish")){
					//if(st.hasMoreTokens())
						//ci.setSharedKey(st.nextToken().getBytes());
					return;
				//}
			}
			return;
		}

		ciphertext = data;		
		data = ci.decrypt(ciphertext);
		if (data==null) 
			if(isReceiver)
				data = cipher;
			else return;
		

		th = new TabHighlighter(tc, s);
     		SwingUtilities.invokeLater(th);
		


		if(isReceiver){
			cu = new ChatUpdater(tp, 
					"<" + from + "> ", tp.getStyle("usay"));
     			SwingUtilities.invokeLater(cu);
			System.out.print(from + "  --> ");
			cl = new ChatLogger(tc, from, from, data);
		}else{
			cu = new ChatUpdater(tp, 
					"<" + tc.getUserName() + "> ", 
					tp.getStyle("isay"));
     			SwingUtilities.invokeLater(cu);
			System.out.print(to + " <--  ");
			cl = new ChatLogger(tc, tc.getUserName(), to, data);
		}
		cu = new ChatUpdater(tp,  data + "\n", tp.getStyle("chat"));
     		SwingUtilities.invokeLater(cu);
		SwingUtilities.invokeLater(cl);
		System.out.println(data);
		
	}
}

/**
 * Handles all automated messages.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 2002-11-26
 */
class AutoReader implements  RMsgHandler {
	TabbedChat tc=null;

        /**
         * Default ctor.
         *
         */
	public AutoReader(){}

	/**
	 * Sets the parent TabbedChat object.
	 *
	 * @param t Parent TabbedChat object.
	 */
	public void setTabs(TabbedChat t){ tc = t;}

        /**
         * Implements handle to handle received messages.
         *
         * @param to The message target.
         * @param from The message sender.
         * @param data The message contents.
         */
	public void handle(String to, String from, String data) {
		JTextPane tp = tc.openBuddy(from);
		TabHighlighter th;
		ChatUpdater cu;
		ChatLogger cl;


		// If we're ignoring AUTO messages from this person, leave!
		if(tc.isIgnored(from, "AUTO")) return;


		th = new TabHighlighter(tc, from);
		SwingUtilities.invokeLater(th);

		cu = new ChatUpdater(tp, "--" + data+"\n", tp.getStyle("auto"));
		cl = new ChatLogger(tc, from, to, data);
		SwingUtilities.invokeLater(cu);
		SwingUtilities.invokeLater(cl);

	}

	
}


/**
 * Manages chats with multiple targets in a single JTabbedPane.
 * Also provides several functions for accessing information on 
 * the active connection.
 *
 * @author Matthew Wronka <wronkm@rpi.edu>
 * @version 0
 */
public class TabbedChat  {
	final protected JTabbedPane tabbedPane = new JTabbedPane();
	final protected ChatReader cr = new ChatReader();
	final protected WallReader wr = new WallReader();
	final protected SystemReader sr = new SystemReader();
	final protected MsgReader mr = new MsgReader();
	final protected AutoReader ar = new AutoReader();

	final protected LinkedList lstChatIgnore = new LinkedList();
	final protected LinkedList lstWallIgnore = new LinkedList();
	final protected LinkedList lstSystemIgnore = new LinkedList();
	final protected LinkedList lstMsgIgnore = new LinkedList();
	final protected LinkedList lstAutoIgnore = new LinkedList();
	final protected LinkedList lstAllIgnore = new LinkedList();

	protected WTalk wt = null;
	protected wtalk.IUserList ul = null;
	protected TabbedConnection m_tc = null;
	protected String m_ConnectionName = null;


	/**
	 * Default ctor. 
	 */
	public TabbedChat() {
		InitTabbedChat();
		//setConnectionType(wtalk.Protocol.USA);
		//setConnectionType(wtalk.Protocol.PMS);
		//setConnectionType(wtalk.Protocol.POP);
	}

	/**
	 * Overloaded ctor.
	 *
	 * @param tc Parent TabbedConnection containing this TabbedChat.
	 * @param name Connection's name.
	 */
	public TabbedChat(TabbedConnection tc, String name) {
		m_tc = tc;
		m_ConnectionName = name;
		wt = new WTalk(m_ConnectionName);
		InitTabbedChat();
	}

	/**
	 * Initilizes TabbedChat
	 */
	public void InitTabbedChat() {
		tabbedPane.setFocusable(false);
		initReaders();
		addListeners();
		addKeyBindings();
	}

	/**
	 * Returns the parent TabbedConnection.
	 *
	 * @return  TabbedConnection containing this TabbedChat.
	 */
	public TabbedConnection getTabbedConnection(){ return m_tc; }

	/**
	 * Returns the current connection name.
	 *
	 * @return Returns the name of the current connection.
	 */
	public String getConnectionName(){ return m_ConnectionName;}

	/**
	 * Sets the protocol used to connect.  This should be a constant
	 * defined in wtalk.Protocol.
	 *
	 * @param i Protocol to use, as defined in wtalk.Protocol.
	 * @see wtalk.WTalk
	 * @see wtalk.Protocol
	 */
	public void setConnectionType(int i) {
		wt.setWTalkProtocol(i);
		registerReaders();
	}

	/**
	 * Initializes message readers.
	 */
	private void initReaders() {
		JChatPanel cp = addTab("System", JChatPanel.SYSTEM_TAB);
		wr.setPane(cp.getJTextPane());
		sr.setPane(cp.getJTextPane());
		wr.setTabs(this);
		sr.setTabs(this);
		cr.setTabs(this);
		mr.setTabs(this);
		ar.setTabs(this);
	}

	/**
	 * Register readers with wtalk.WTalk object.
	 */
	private  void registerReaders() {
		if(wt.getWTalkProtocol()==null) {
			System.err.println("WTalk Protocol is UNDEF!");
			return;
		}

		wt.getWTalkProtocol().setReceivedMessageHandler(cr);
		wt.getWTalkProtocol().setChatMessageHandler(cr);
		wt.getWTalkProtocol().setPrivateMessageHandler(mr);
		wt.getWTalkProtocol().setWallMessageHandler(wr);
		wt.getWTalkProtocol().setSystemMessageHandler(sr);
		wt.getWTalkProtocol().setAutoMessageHandler(ar);
		wt.getWTalkProtocol().setUserListHandler(ul);
	}

	/**
	 * Sets the object to register with WTalkProtocols that will
	 * implement IUserList to support GUI representation of buddy lists.
	 *
	 * @param u Object that will be used to manage user lists.
	 */
	public void setUserList(wtalk.IUserList u) { 
		ul = u; 
		wt.getWTalkProtocol().setUserListHandler(ul);
	}

	/**
	 * Adds event listeners to tabs to facilitate highlighting of 
	 * tabs which do not have focus when they receive messages.
	 */
	private void addListeners() {
		tabbedPane.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				tabbedPane.setForegroundAt(tabbedPane.getSelectedIndex(), DmcColors.TAB_NORMAL);
			}
		});
	}

	 /**
	 * Adds keybindigns to facilitate switching conversations.
	 */
	private void addKeyBindings() {
		tabbedPane.getInputMap(
			JComponent.WHEN_IN_FOCUSED_WINDOW).
			 put(KeyStroke.getKeyStroke("ctrl H"), "next-conversation") ;
		tabbedPane.getInputMap(
			 JComponent.WHEN_IN_FOCUSED_WINDOW).
			put(KeyStroke.getKeyStroke("ctrl L"), "prev-conversation") ;

		Action PrevConversation = new AbstractAction(){
			public void actionPerformed(ActionEvent e){
				int i=tabbedPane.getSelectedIndex();
				if(i==tabbedPane.getTabCount()-1)
					tabbedPane.setSelectedIndex(0);
				else
					tabbedPane.setSelectedIndex(i+1);
			}
		 };

		Action NextConversation = new AbstractAction(){
			public void actionPerformed(ActionEvent e){
				int i=tabbedPane.getSelectedIndex();
				if(i==0)
					tabbedPane.setSelectedIndex(
						tabbedPane.getTabCount()-1);
				else
					tabbedPane.setSelectedIndex(i-1);

			}
		};


		tabbedPane.getActionMap().put("next-conversation", NextConversation) ;
		tabbedPane.getActionMap().put("prev-conversation", PrevConversation) ;
	}


	/**
	 * Returns the GUI representation of a TabbedChat.
	 *
	 * @return Returns GUI  representation of this object.
	 */
	public JTabbedPane getUI() {
		return tabbedPane;
	}

	/**
	 * Returns the index of the tab with caption 's' in tabbedPane.
	 *
	 * @param s Caption of target tag.
	 * @return Index of tag with Caption 's', or -1 on failure.
	 */
	public int indexOfTab(String s) {
		for(int i = tabbedPane.getTabCount(); --i >= 0 ;) {
			if(s.equalsIgnoreCase(tabbedPane.getTitleAt(i)))
				return i;
		}
		return -1;
	}

	/**
	 * Returns a ChatInputHandler associated with the JChatPanel
	 * in charge of <em>room</em>.
	 *
	 * @param room	Room for which we want to get a ChatInputHandler.
	 * @return ChatInputHandler for <em>room</em>
	 */
	public ChatInputHandler getInputPanel(String room) {
		int i = getUI().indexOfTab(room); 
		return ((JChatPanel)(getUI().getComponentAt(i))).getChatInputHandler();
	}

	/**
	 * Returns  the user name as reported by the WTalkProtocol.
	 *
	 * @return User name assocaited with the active connection.
	 */
	public String getUserName() {
		return wt.getWTalkProtocol().getName();
	}

	/**
	 * Tells the WTalkProtocol to  request a new name, <em>name</em>.
	 * There is no guarantee that this name will be granted
	 *
	 * @param name Name to request.
	 */
	public void setName(String name) {
		wt.getWTalkProtocol().setName(name);
	}

	/**
	 * Tells the WTalkProtocol to use the given password when trying
	 * to authenticate.
	 *
	 * @param passwd Password to use to authenticate.
	 */
	public void setPassword(String passwd) {
		wt.getWTalkProtocol().setPassword(passwd);
	}


	/**
	 * Connects to <em>server:port</em> using <em>login</em> as a handle.
	 *
	 * @param server Host to connect to.
	 * @param port Port to connect to on host.
	 * @param login Name to login with.
	 * @throws IOException Throws IOException when the WTalkProtocol cannot
	 *	connect.  For instance, the host is down, or cannot be resolved.
	 */
	public void connect(String server, int port, String login) throws IOException{
		wt.getWTalkProtocol().connect(server, port, login);
	}

	/**
	 * Adds a new tab to the TabbedChat with caption <em>name</em>.
	 *	
 	 * @param name Caption for tab. 
	 * @param type Type of JChatPanel to create.  
	 *	These are defined in JChatPanel.
	 * @return JChatPanel that belongs to the new tab.
	 * @see JChatPanel
	 */
	public JChatPanel addTab(String name, int type) {
		JChatPanel pane = new JChatPanel(this, wt, type!=0?name:null);
		
		pane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		tabbedPane.addTab(name, pane);
		tabbedPane.setForegroundAt(tabbedPane.indexOfTab(name), DmcColors.TAB_NORMAL);

		return pane;
	}

	/**
	 * Returns the JTextPane used by the JChatPanel displaying the 
	 *	chat with <em>name</em>.  If none exists, a new tab is added
	 *	which contains a JChatPanel to use in communications with
	 *	this buddy.
	 *
	 * @param name The handle of the user whose JChatPanel's JTextPane
	 *	is to be returned.
	 * @return JTextPane used by the JChatPanel that represents the chat
	 *	 with <em>name</em>.
	 */
	public JTextPane openBuddy(String name) {
		int i = indexOfTab(name);
		JTextPane tp=null;

		if(i>=0) 
			tp = ((JChatPanel)(getUI().getComponentAt(i))).getJTextPane(); 
		else
			tp = addTab(name, JChatPanel.CHAT_TAB).getJTextPane();

		return tp;
	}

        /**
         * Returns whether or not <em>name</em> is being ignored.
         *
         * @param name The name of the user in question.
         * @param type The type of conversation in question.
	 * @return Returns <em>true</em> if <em>name</em> is ignored.
         */
	public boolean isIgnored ( String name, String type) {
		if(lstAllIgnore.contains(name))
			return true;

		if(type==null) return false;
		if(type.equalsIgnoreCase("MSG")) {
			if(lstMsgIgnore.contains(name)) return true;
		} else if(type.equalsIgnoreCase("CHAT")) {
			if(lstChatIgnore.contains(name)) return true;
		} else if(type.equalsIgnoreCase("SYSTEM")) {
			if(lstSystemIgnore.contains(name)) return true;
		} else if(type.equalsIgnoreCase("AUTO")) {
			if(lstAutoIgnore.contains(name)) return true;
		} else if(type.equalsIgnoreCase("WALL")) {
			if(lstWallIgnore.contains(name)) return true;
		}

		return false;
	}

	/**
	 * Ignores <em>name</em> from conversations of type <em>type</em>.
	 *
	 * @param name The name of the user to ignore.
	 * @param type The type of conversation to ignore name in.  
	 *	Defaults to "ALL".  Valid values are "ALL" "MSG" 
	 *	"CHAT" "SYSTEM" "WALL" and  "AUTO".
	 */
	public void addIgnore ( String name, String type) {
                if(type==null) type = "ALL";
                if(type.equalsIgnoreCase("MSG")) {
                        lstMsgIgnore.add(name);
                } else if(type.equalsIgnoreCase("CHAT")) {
                        lstChatIgnore.add(name);
                } else if(type.equalsIgnoreCase("SYSTEM")) {
                        lstSystemIgnore.add(name);
                } else if(type.equalsIgnoreCase("AUTO")) {
                        lstAutoIgnore.add(name);
                } else if(type.equalsIgnoreCase("WALL")) {
                        lstWallIgnore.add(name);
                } else {
                        lstAllIgnore.add(name);
		}
	}


        /**
         * Stops ignoring <em>name</em> from conversations 
	 *	of type <em>type</em>.
         *
         * @param name The name of the user being ignored.
         * @param type The type of conversation where name is ignored.  
	 *	Defaults to "ALL".  Valid values are "ALL" "MSG" 
	 *	"CHAT" "SYSTEM" "WALL" and  "AUTO".
         */
	public void delIgnore ( String name, String type) {
                if(type==null) type = "ALL";
                if(type.equalsIgnoreCase("MSG")) {
                        lstMsgIgnore.remove(name);
                } else if(type.equalsIgnoreCase("CHAT")) {
                        lstChatIgnore.remove(name);
                } else if(type.equalsIgnoreCase("SYSTEM")) {
                        lstSystemIgnore.remove(name);
                } else if(type.equalsIgnoreCase("AUTO")) {
                        lstAutoIgnore.remove(name);
                } else if(type.equalsIgnoreCase("WALL")) {
                        lstWallIgnore.remove(name);
                } else {
                        lstAllIgnore.remove(name);
                }
	}

	/**
 	 * Show ignored users.
 	 *
	 * @return String containing ignored users.
	 */

	public String showIgnored () {
		String rval="";

		rval += "[ALL]    " + lstAllIgnore + "\n";
		rval += "[MSG]    " + lstMsgIgnore + "\n";
		rval += "[CHAT]   " + lstChatIgnore + "\n";
		rval += "[SYSTEM] " + lstSystemIgnore + "\n";
		rval += "[AUTO]   " + lstAutoIgnore + "\n";
		rval += "[WALL]   " + lstWallIgnore + "\n";
		return rval;
	}
}
