/** ReceivedListener.java
 *
 * By: Ken Hampson -- November, 2002
 *
 * This file provides an implementation of the WTalkProtocol interface. It facilitates the communication between the client and server
 * in the form of parsing messages received from the server. It also serves to
 * connect the GUI with actual back-end functionality.
 */

package wtalk.protocols.pms;

// various imports
import wtalk.RMsgHandler;
import wtalk.IUserList;
import java.io.*;
import java.util.*;

class ReceivedListener extends Thread{
  // Message type codes
  static final int SEND_MESSAGE = 1;
  static final int RESERVED_2 = 2;
  static final int QUERY_STATUS_INFO = 3;
  static final int RETURN_STATUS_INFO = 4;
  static final int SIGN_ON = 5;
  static final int SIGN_OFF = 6;
  static final int GET_AWAY_MESSAGE = 7;
  static final int RETURN_AWAY_MESSAGE = 8;
  static final int POST_AWAY_MESSAGE = 9;
  static final int BAN_USER = 10;
  static final int UNBAN_USER = 11;
  /* 12 and 13 are currently unused */
  static final int SET_BUDDY_LIST = 14;
  static final int ADD_BUDDY = 15;
  static final int REMOVE_BUDDY = 16;
  static final int REMOVE_AWAY_MESSAGE = 17;
  static final int RECEIVE_AWAY_MESSAGE = 18;

  // General codes
  static final int UNKNOWN_MESS =	-2;
  static final int USER_NOT_FOUND = -3;
  static final int UNKNOWN_PASS = -4;
  static final int SIGN_OFF_FAILED = -5;
  static final int TARGET_OFFLINE = -6;
  static final int UPDATE_FAILURE = -7;
  static final int SEND_FAILED = -8;

  // Success codes
  static final int VALIDATED = 104;
  static final int OLD_DATA_DELETED = 105;
  static final int UPDATE_SUCCESSFUL = 106;

  // Connection-related codes
  static final int SESSION_TERMINATED = 100;
  static final int CONNECTION_DENIED = 101;
  static final int CONNECTION_FAILED = 102;
  static final int CONNECTED_SUCCESFULLY = 103;

  // Designates the various levels of debug -- 0 is none
  static final int DEBUG = 1;

  DataInputStream mIn = null;

  private RMsgHandler mRmh = null;						// generic message handler
  private RMsgHandler mRwh = null;						// handles wall messages if !null
  private RMsgHandler mRsh = null; 					// handles system messages if !null
  private RMsgHandler mRph = null; 					// handles private messages if !null
  private RMsgHandler mRah = null; 					// handles auto messages if !null
  private IUserList mUL = null;

  private String connectionName = "";

  // default constructor
  public ReceivedListener(){}

  public void  setConnectionName(String s) { connectionName = s; }

  /**Sets the socket on which the client should listen
   * @author Matt Wronka
   * @param n The socket to listen on
   */
  public void ListenTo(DataInputStream n) {
    mIn = n;
  }

  /**The main thread of execution for the listener
   * @author Matt Wronka
   */
  public void run(){
    while(true) {
      try {
        readMsg(mIn);
      }
      catch (IOException e) {
      }
      catch (NumberFormatException e) {
      }
    }
  }


  /**
   * Registers an object that implements wtalk.RMsgHandler
   * which whill handle all other unhandled messages.
   *
   * @param Default message handler.
   */
  public void ReportTo(RMsgHandler r) {
    mRmh = r;
  }

  /** 
   * Registers an object that implements wtalk.RMsgHandler
   * which whill handle all wall (brodcast) messages.
   *
   * @param Wall message handler.
   */
  public void ReportWallTo(RMsgHandler r){
    mRwh = r;
  }

  /** 
   * Registers an object that implements wtalk.RMsgHandler
   * which whill handle all System messages.
   *
   * @param System message handler.
   */
  public void ReportSystemTo(RMsgHandler r) {
    mRsh = r;
  }

  /** 
   * Registers an object that implements wtalk.RMsgHandler
   * which whill handle all person->person messages.
   *
   * @param Private message handler.
   */
  public void ReportPrivateTo(RMsgHandler r) {
    mRph = r;
  }
  
  /** 
   * Registers an object that implements wtalk.RMsgHandler
   * which whill handle all automatic messages, such as away messages.
   *
   * @param Auto-message handler.
   */
  public void ReportAutoTo(RMsgHandler r) {
    mRah = r;
  }

  /**
   * Sets the call-back for functions that manipulate the user list.
   * This is necessary to retrieve the buddy list at sign-on.
   */
  public void UserList(IUserList ul) {
    mUL=ul;
  }

  /**Parse out the individual components of the message header.
   * @author Ken Hampson
   * @param dis A DataInputStream mapped to the socket from which the message header is to be read
   * @throws IOException When there is a problem reading from the socket
   */
  public void readMsg(DataInputStream dis) throws IOException, NumberFormatException {
    int version = 0x0000;                                // The version of the protocol being used (8 bits)
    int msgType = 0x0000;                                // The type of the message being sent (8 bits)
    int time = 0;                                        // The time the message was sent
    int messageSize = 0;                                 // The size of the message mIn bytes
    int dataSize = 0;					 // The size of the data associated with this message
    byte[] messageDataBuffer = null;			 // Buffer to hold the message data
    String message = null;                               // A more detailed status message

    // read in the protocol version
    version = (int)dis.readByte();

    // read in the type of message
    msgType = (int)dis.readByte();

    // read in the length of the data coming mIn
    dataSize = (int)dis.readShort();

    // read in the time the message was sent
    time = dis.readInt();

    if (DEBUG >= 1){
      System.out.println("\nheader> Version = " + version + "; msgType = " + msgType + "; dataSize = " + dataSize);
    }

    /* Decide how we should proceed based on the message type */
    switch(msgType){
      case SEND_MESSAGE:
      case RECEIVE_AWAY_MESSAGE:
        // Used in mutliple cases
        String name = null;                                 // The user's name
        int nameSize = 0;                                   // The size of the user's name, mIn bytes

        int senderNameSize = 0;                             // The size of the sender's name, mIn bytes
        int recipientNameSize = 0;                          // The size of the recipient's name, mIn bytes
        int chatIDSize = 0;                                 // the size of the chat ID, mIn bytes
        int numRecip = 0;                                   // The number of recipients for the message

        byte[] tmpBuffer = null;                            // a temporary buffer to hold bytes read from the input stream

        String senderName = null;                           // The name of the user who sent this message
        String recipientName = null;                        // The name of the current recipient of this message
        String chatID = null;                               // The ID of the chat associated with this message

        // read in the size of the sender name and the name that follows
        senderNameSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[senderNameSize];
        dis.read(tmpBuffer, 0, senderNameSize);
        senderName = new String(tmpBuffer);
        dis.readByte();

        // ** For now only check for a single recipient
        // read in the number of recips and toss
        dis.readByte();

        // read in the size of the receiver name and the name that follows
        recipientNameSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[recipientNameSize];
        dis.read(tmpBuffer, 0, recipientNameSize);
        recipientName = new String(tmpBuffer);
        dis.readByte();

        // read in the size of the chatID and the ID itself
        chatIDSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[chatIDSize];
        dis.read(tmpBuffer, 0, chatIDSize);
        chatID = new String(tmpBuffer);
        dis.readByte();

        // read in the size of the data to follow
        messageSize = (int)dis.readShort() - 1;

        // allocate enough memory for the data buffer
        messageDataBuffer = new byte[messageSize];

        // read the data itself into the data buffer
        dis.read(messageDataBuffer, 0, messageSize);

        dis.readByte();

        String dataString = new String(messageDataBuffer);

        if (DEBUG >= 1){
          System.out.println("SEND_MESSAGE> senderName = " + senderName + "; recipientNameSize = " + recipientNameSize +
                             "; recipientName = " + recipientName + "; messageDataBuffer = " + dataString);
        }

        // distinguish between normal and auto-response message
        if(mRph != null){
          if (msgType == SEND_MESSAGE)
            mRph.handle(null , senderName, dataString);
          else if(mRah != null) {
            // auto response
            mRah.handle(null, senderName, dataString);
	  }
        }

        else if(mRmh != null){
          if (msgType == SEND_MESSAGE)
            mRmh.handle(null , senderName, dataString);
          else
            // auto response
            mRmh.handle(null, null, dataString);
        }

        break;

        // currently reserved
      case 2:
        break;

        // not currently implemented in full in other areas
      case QUERY_STATUS_INFO:
        int queryNameSize = 0;                                 // The size of the name of the user for which to retrieve the status info
        String queryName = null;                               // The name of the user for which to retrieve the status info

        // read in the size of the query name and the name that follows
        queryNameSize = (int)dis.readShort() - 1;
        tmpBuffer = new byte[queryNameSize];
        dis.read(tmpBuffer, 0, queryNameSize);
        queryName = new String(tmpBuffer);
        dis.readByte();

        if (DEBUG >= 1){
          System.out.println("QUERY_STATUS_INFO> queryName = " + queryName);
        }

        break;

        // not currently implemented in full in other areas
      case RETURN_STATUS_INFO:
        int phoneNumberSize = 0;                               // The length of the user's phone number
        int eMailSize = 0;                                     // The length of the user's e-mail address
        int majorNameSize = 0;                                 // The length of the user's major name
        int locationSize = 0;                                  // The length of the user's location name
        int awayTime = 0;                                      // The user's away time mIn seconds (0 indicates not away)
        int idleTime = 0;                                      // The user's idle time mIn seconds (0 indicates not idle)

        String phoneNumber = null;                             // The user's phone number
        String eMail = null;                                   // The user's e-mail address
        String majorName = null;                               // The user's major
        String location = null;                                // The user's location

        // read in the size of the name and the name itself that follows
        nameSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[nameSize];
        dis.read(tmpBuffer, 0, nameSize);
        name = new String(tmpBuffer);
        dis.readByte();

        // read in the size of the phone number and the number itself that follows
        phoneNumberSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[phoneNumberSize];
        dis.read(tmpBuffer, 0, phoneNumberSize);
        phoneNumber = new String(tmpBuffer);
        dis.readByte();

        // read in the size of the email address and the address itself that follows
        eMailSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[eMailSize];
        dis.read(tmpBuffer, 0, eMailSize);
        eMail = new String(tmpBuffer);
        dis.readByte();

        // read in the size of the major name and the name itself that follows
        majorNameSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[majorNameSize];
        dis.read(tmpBuffer, 0, majorNameSize);
        majorName = new String(tmpBuffer);
        dis.readByte();

        // read in the size of the location and the location itself that follows
        locationSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[locationSize];
        dis.read(tmpBuffer, 0, locationSize);
        location = new String(tmpBuffer);
        dis.readByte();

        // read in the user's away time
        awayTime = dis.readInt();

        // read in the user's idle time
        idleTime = dis.readInt();

        if (DEBUG >= 1){
          System.out.println("RETURN_STATUS_INFO> name = " + name + "; phoneNumber = " + phoneNumber + "; eMail = " + eMail +
                             "; majorName = " + majorName + "; location = " + location + "; awayTime = " +
                             awayTime + "; idleTime = " + idleTime);
        }

        // ** call the displayStatus fctn here
        break;

        // not currently implemented in full in other areas
      case RETURN_AWAY_MESSAGE:
        int awayMessageSize = 0;                                 // The size of the away message
        String awayMessage = null;                               // The contents of the away message

        // read in the size of the name and the name itself that follows
        nameSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[nameSize];
        dis.read(tmpBuffer, 0, nameSize);
        name = new String(tmpBuffer);
        dis.readByte();

        // read in the size of the away message and the message itself that follows
        awayMessageSize = (int)dis.readByte() - 1;
        tmpBuffer = new byte[awayMessageSize];
        dis.read(tmpBuffer, 0, awayMessageSize);
        awayMessage = new String(tmpBuffer);
        dis.readByte();

        if (DEBUG >= 1){
          System.out.println("RETURN_AWAY_MESSAGE> name = " + name + "; awayMessage = " + awayMessage);
        }

        // ** Call displayAwayMessage fctn
        break;

        // the various success cases
      case VALIDATED:
      case CONNECTED_SUCCESFULLY:
      case UPDATE_SUCCESSFUL:

        // read in the size of the message
        messageSize = (int)dis.readByte() - 1;

        // get the message if one exists
        if(messageSize > 0){
          tmpBuffer = new byte[messageSize];
          dis.read(tmpBuffer, 0, messageSize);
          message = new String(tmpBuffer);
        }

        // soak up the message size
        dis.readByte();

        if (DEBUG >= 1){
          System.out.println("SUCCESS> message = " + message);
        }

        // send the message to the system tab on the client
        if (mRsh != null)
          mRsh.handle(null, null, message);

        // try the default handler if the system handler is null
        else if (mRmh != null)
          mRmh.handle(null, null, message);
        break;


        // the various error cases
      case UNKNOWN_MESS:
      case USER_NOT_FOUND:
      case UNKNOWN_PASS:
      case SIGN_OFF_FAILED:
      case TARGET_OFFLINE:
      case UPDATE_FAILURE:
      case SEND_FAILED:
      case SESSION_TERMINATED:
      case CONNECTION_DENIED:
      case CONNECTION_FAILED:
      case OLD_DATA_DELETED:

        // read in the size of the message
        messageSize = (int)dis.readByte() - 1;

        // get the message if one exists
        if(messageSize > 0){
          tmpBuffer = new byte[messageSize];
          dis.read(tmpBuffer, 0, messageSize);
          message = new String(tmpBuffer);
        }

        // soak up the message size
        dis.readByte();

        if (DEBUG >= 1){
          System.out.println("ERROR> message = " + message);
        }

        // send the message to the system tab on the client
        if (mRsh != null)
          mRsh.handle(null, null, message);

        // try the default handler if the system handler is null
        else if (mRmh != null)
          mRmh.handle(null, null, message);
        break;

        // format of string -->  groupname:user:user:user<NL> ...
      case SET_BUDDY_LIST:
        byte[] bfr = new byte[dataSize];
        dis.readByte();
        dis.read(bfr, 0, dataSize);
        String prevToken = null;					// The previous token
        String currGroup = null;					// The current group we're working with

        StringTokenizer st = new StringTokenizer(new String(bfr), ":\n\0", true);			// String tokenizer; delimits on :, newline, carriage return

        // extract the groupnames and usernames from the string and add them to the buddy list tree
        while (st.hasMoreTokens()) {
          String currToken = st.nextToken();				// The current token we're working with
          System.out.println(currToken);

          if (currToken.equals(":")){
            if  (currGroup == null){
              currGroup = new String(prevToken);
              // prevToken is a group name, so add the group to the buddy list
              mUL.addGroup(currGroup, connectionName);
              System.err.println("\tShould be adding " + currGroup);
            } else {
              mUL.addUser(prevToken, currGroup, connectionName);
              System.err.println("\tShould be adding [" + prevToken + "] to [" + currGroup + "]");
            }
          } else if ((currToken.equals("\n") || currToken.equals("\0"))  && !prevToken.equals("")){
            mUL.addUser(prevToken, currGroup, connectionName);
            System.err.println("\tShould be adding [" + prevToken + "] to [" + currGroup + "]");
            // signifies the end of a group
            currGroup = null;
          }
          prevToken=currToken;
        }
    }
  }
}
